!***********************************************************************
!*   
!*   Licensed Materials - Property of IBM.
!*   IBM XL Fortran Advanced Edition V10.1 for Linux (5724-M17)
!*   Copyright IBM Corp. 2003, 2006.
!*   All Rights Reserved.
!*   US Government Users Restricted Rights - 
!*   Use, duplication or disclosure restricted by 
!*   GSA ADP Schedule Contract with IBM Corp.
!*
!***********************************************************************
!
!            NOTICE TO USERS OF THE SOURCE CODE EXAMPLES
!
! THE SOURCE CODE EXAMPLES PROVIDED BY IBM ARE ONLY INTENDED TO ASSIST IN THE
! DEVELOPMENT OF A WORKING SOFTWARE PROGRAM.  THE SOURCE CODE EXAMPLES DO NOT
! FUNCTION AS WRITTEN:  ADDITIONAL CODE IS REQUIRED.  IN ADDITION, THE SOURCE
! CODE EXAMPLES MAY NOT COMPILE AND/OR BIND SUCCESSFULLY AS WRITTEN.
!
! INTERNATIONAL BUSINESS MACHINES CORPORATION PROVIDES THE SOURCE CODE
! EXAMPLES, BOTH INDIVIDUALLY AND AS ONE OR MORE GROUPS, "AS IS" WITHOUT
! WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT
! LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
! PARTICULAR PURPOSE.  THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE
! OF THE SOURCE CODE EXAMPLES, BOTH INDIVIDUALLY AND AS ONE OR MORE GROUPS,
! IS WITH YOU.  SHOULD ANY PART OF THE SOURCE CODE EXAMPLES PROVE
! DEFECTIVE, YOU (AND NOT IBM OR AN AUTHORIZED RISC System/6000* WORKSTATION
! DEALER) ASSUME THE ENTIRE COST OF ALL NECESSARY SERVICING, REPAIR OR
! CORRECTION.
!
! IBM does not warrant that the contents of the source code examples, whether
! individually or as one or more groups, will meet your requirements or that
! the source code examples are error-free.
!
! IBM may make improvements and/or changes in the source code examples at
! any time.
!
! Changes may be made periodically to the information in the source code
! examples; these changes may be reported in new editions of the examples.
!
! References in the source code examples to IBM products, programs, or
! services do not imply that IBM intends to make these available in all
! countries in which IBM operates.  Any reference to an IBM licensed
! program in the source code examples is not intended to state or imply
! that only IBM's licensed program may be used.  Any functionally equivalent
! program may be used.
!
! * RISC System/6000 is a trademark of International Business Machines
!   Corporation.

!  Sample routines to clear and query the exception indicator bits in
!  the floating point status and control register.

!  Compile with 'xlf -c check_fpscr.f'.


        subroutine reset_exception_bits
!  Clear all of the exception indicator bits in the floating point
!  status and control register.

        include 'fpdc.h'

!  get the current value of the bits
        call fpgets( fpstat )

!  clear all of the exception indication bits
        fpstat(fpfx) = .FALSE.  ! clear exception summary
        fpstat(fpox) = .FALSE.  ! clear overflow
        fpstat(fpux) = .FALSE.  ! clear underflow
        fpstat(fpzx) = .FALSE.  ! clear zero divide
        fpstat(fpxx) = .FALSE.  ! clear inexact
!    clear the bits for specific invalid operations (the fpvx bit is the
!    logical OR of these bits, so clearing these bits will clear fpvx)
        fpstat(fpvxsnan) = .FALSE.  ! clear invalid operation (NaNS)
        fpstat(fpvxisi) = .FALSE.   ! clear invalid operation (INF-INF)
        fpstat(fpvxidi) = .FALSE.   ! clear invalid operation (INF/INF)
        fpstat(fpvxzdz) = .FALSE.   ! clear invalid operation (0/0)
        fpstat(fpvximz) = .FALSE.   ! clear invalid operation (INF*0)
        fpstat(fpvxvc) = .FALSE.    ! clear invalid operation (inv. compare)

!  set the register to the new value
        call fpsets( fpstat )
        end


        subroutine query_exception_bits
!  Report on all floating point exceptions that have occurred by
!  checking the floating point status and control register.

        include 'fpdc.h'

!  get the current value of the bits
        call fpgets( fpstat )

!  the fpfx bit will be set if any exceptions occurred
        if ( fpstat(fpfx) .eqv. .TRUE. ) then
          print *, 'Floating point exceptions have occurred:'
          if ( fpstat(fpvx) .eqv. .TRUE. ) then
            print *, '    invalid operation'
!    additional checks could be inserted here to determine which types of
!    invalid operation occurred
          endif
          if ( fpstat(fpox) .eqv. .TRUE. ) then
            print *, '    overflow'
          endif
          if ( fpstat(fpux) .eqv. .TRUE. ) then
            print *, '    underflow'
          endif
          if ( fpstat(fpzx) .eqv. .TRUE. ) then
            print *, '    divide by zero'
          endif
          if ( fpstat(fpxx) .eqv. .TRUE. ) then
            print *, '    inexact'
          endif
        endif
        end


        block data
        include 'fpdc.h'
        include 'fpdt.h'
        end
